/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for licence. */
#ifndef _NIF_ENUMS_H_
#define _NIF_ENUMS_H_

#include "nif_basic_types.h"

namespace Niflib {

/* Template converters for Enum Data Types */
template <typename T> std::string EnumToString(T value);
template <typename T> T StringToEnum(const std::string& value);

/*!
 * The type of force?  May be more valid values.
 */
typedef enum ForceType : uint {
	FORCE_PLANAR = 0, /*!< FORCE_PLANAR */
	FORCE_SPHERICAL = 1, /*!< FORCE_SPHERICAL */
	FORCE_UNKNOWN = 2, /*!< FORCE_UNKNOWN */
} ForceType;

template <> std::string EnumToString<ForceType>(ForceType value);
template <> ForceType StringToEnum<ForceType>(const std::string& value);
void NifStream( ForceType & val, istream& in, uint version = 0 );
void NifStream( ForceType const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, ForceType const & val );

/*!
 * An unsigned 32-bit integer, describing the color depth of a texture.
 */
typedef enum PixelLayout : uint {
	PIX_LAY_PALETTISED = 0, /*!< Texture is in 8-bit paletized format. */
	PIX_LAY_HIGH_COLOR_16 = 1, /*!< Texture is in 16-bit high color format. */
	PIX_LAY_TRUE_COLOR_32 = 2, /*!< Texture is in 32-bit true color format. */
	PIX_LAY_COMPRESSED = 3, /*!< Texture is compressed. */
	PIX_LAY_BUMPMAP = 4, /*!< Texture is a grayscale bump map. */
	PIX_LAY_DEFAULT = 5, /*!< Use default setting. */
} PixelLayout;

template <> std::string EnumToString<PixelLayout>(PixelLayout value);
template <> PixelLayout StringToEnum<PixelLayout>(const std::string& value);
void NifStream( PixelLayout & val, istream& in, uint version = 0 );
void NifStream( PixelLayout const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, PixelLayout const & val );

/*!
 * An unsigned 32-bit integer, describing how vertex colors influence
 * lighting.
 */
typedef enum LightMode : uint {
	LIGHT_MODE_EMISSIVE = 0, /*!< Emissive. */
	LIGHT_MODE_EMI_AMB_DIF = 1, /*!< Emissive + Ambient + Diffuse. (Default) */
} LightMode;

template <> std::string EnumToString<LightMode>(LightMode value);
template <> LightMode StringToEnum<LightMode>(const std::string& value);
void NifStream( LightMode & val, istream& in, uint version = 0 );
void NifStream( LightMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, LightMode const & val );

/*!
 * An unsigned 32-bit integer, describing how mipmaps are handled in a
 * texture.
 */
typedef enum MipMapFormat : uint {
	MIP_FMT_NO = 0, /*!< Texture does not use mip maps. */
	MIP_FMT_YES = 1, /*!< Texture uses mip maps. */
	MIP_FMT_DEFAULT = 2, /*!< Use default setting. */
} MipMapFormat;

template <> std::string EnumToString<MipMapFormat>(MipMapFormat value);
template <> MipMapFormat StringToEnum<MipMapFormat>(const std::string& value);
void NifStream( MipMapFormat & val, istream& in, uint version = 0 );
void NifStream( MipMapFormat const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, MipMapFormat const & val );

/*!
 * An unsigned 32-bit integer, describing how transparency is handled in
 * a texture.
 */
typedef enum AlphaFormat : uint {
	ALPHA_NONE = 0, /*!< No alpha blending; the texture is fully opaque. */
	ALPHA_BINARY = 1, /*!< Texture is either fully transparent or fully opaque.  There are no partially transparent areas. */
	ALPHA_SMOOTH = 2, /*!< Full range of alpha values can be used from fully transparent to fully opaque including all partially transparent values in between. */
	ALPHA_DEFAULT = 3, /*!< Use default setting. */
} AlphaFormat;

template <> std::string EnumToString<AlphaFormat>(AlphaFormat value);
template <> AlphaFormat StringToEnum<AlphaFormat>(const std::string& value);
void NifStream( AlphaFormat & val, istream& in, uint version = 0 );
void NifStream( AlphaFormat const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, AlphaFormat const & val );

/*!
 * Specifies the availiable texture filter modes.  That is, the way
 * pixels within a texture are blended together when textures are
 * displayed on the screen at a size other than their original
 * dimentions.
 */
typedef enum TexFilterMode : uint {
	FILTER_NEAREST = 0, /*!< Simply uses the nearest pixel.  Very grainy. */
	FILTER_BILERP = 1, /*!< Uses bilinear filtering. */
	FILTER_TRILERP = 2, /*!< Uses trilinear filtering. */
	FILTER_NEAREST_MIPNEAREST = 3, /*!< Uses the nearest pixel from the mipmap that is closest to the display size. */
	FILTER_NEAREST_MIPLERP = 4, /*!< Blends the two mipmaps closest to the display size linearly, and then uses the nearest pixel from the result. */
	FILTER_BILERP_MIPNEAREST = 5, /*!< Uses the closest mipmap to the display size and then uses bilinear filtering on the pixels. */
} TexFilterMode;

template <> std::string EnumToString<TexFilterMode>(TexFilterMode value);
template <> TexFilterMode StringToEnum<TexFilterMode>(const std::string& value);
void NifStream( TexFilterMode & val, istream& in, uint version = 0 );
void NifStream( TexFilterMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, TexFilterMode const & val );

/*!
 * The motion type. Determines quality of motion?
 */
typedef enum MotionQuality : byte {
	MO_QUAL_MOVING = 0, /*!< Moving */
	MO_QUAL_FIXED = 1, /*!< Fixed */
	MO_QUAL_KEYFRAMED = 2, /*!< Keyframed */
	MO_QUAL_MOVING2 = 3, /*!< Moving(?) */
	MO_QUAL_MOVING3 = 4, /*!< Moving(?) */
	MO_QUAL_CRITICAL = 5, /*!< Critical */
	MO_QUAL_BULLET = 6, /*!< Bullet */
	MO_QUAL_USER = 7, /*!< User */
	MO_QUAL_NULL = 8, /*!< Null */
} MotionQuality;

template <> std::string EnumToString<MotionQuality>(MotionQuality value);
template <> MotionQuality StringToEnum<MotionQuality>(const std::string& value);
void NifStream( MotionQuality & val, istream& in, uint version = 0 );
void NifStream( MotionQuality const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, MotionQuality const & val );

/*!
 * Sets mesh color in Oblivion Construction Set.  Anything higher than 57
 * is also null.
 */
typedef enum OblivionLayer : uint {
	OL_UNIDENTIFIED = 0, /*!< Unidentified (white) */
	OL_STATIC = 1, /*!< Static (red) */
	OL_ANIM_STATIC = 2, /*!< AnimStatic (magenta) */
	OL_TRANSPARENT = 3, /*!< Transparent (light pink) */
	OL_CLUTTER = 4, /*!< Clutter (light blue) */
	OL_WEAPON = 5, /*!< Weapon (orange) */
	OL_PROJECTILE = 6, /*!< Projectile (light orange) */
	OL_SPELL = 7, /*!< Spell (cyan) */
	OL_BIPED = 8, /*!< Biped (green) Seems to apply to all creatures/NPCs */
	OL_TREES = 9, /*!< Trees (light brown) */
	OL_PROPS = 10, /*!< Props (magenta) */
	OL_WATER = 11, /*!< Water (cyan) */
	OL_TRIGGER = 12, /*!< Trigger (light grey) */
	OL_TERRAIN = 13, /*!< Terrain (light yellow) */
	OL_TRAP = 14, /*!< Trap (light grey) */
	OL_NONCOLLIDABLE = 15, /*!< NonCollidable (white) */
	OL_CLOUD_TRAP = 16, /*!< CloudTrap (greenish grey) */
	OL_GROUND = 17, /*!< Ground (none) */
	OL_PORTAL = 18, /*!< Portal (green) */
	OL_STAIRS = 19, /*!< Stairs (white) */
	OL_CHAR_CONTROLLER = 20, /*!< CharController (yellow) */
	OL_AVOID_BOX = 21, /*!< AvoidBox (dark yellow) */
	OL_UNKNOWN1 = 22, /*!< ? (white) */
	OL_UNKNOWN2 = 23, /*!< ? (white) */
	OL_CAMERA_PICK = 24, /*!< CameraPick (white) */
	OL_ITEM_PICK = 25, /*!< ItemPick (white) */
	OL_LINE_OF_SIGHT = 26, /*!< LineOfSight (white) */
	OL_PATH_PICK = 27, /*!< PathPick (white) */
	OL_CUSTOM_PICK_1 = 28, /*!< CustomPick1 (white) */
	OL_CUSTOM_PICK_2 = 29, /*!< CustomPick2 (white) */
	OL_SPELL_EXPLOSION = 30, /*!< SpellExplosion (white) */
	OL_DROPPING_PICK = 31, /*!< DroppingPick (white) */
	OL_OTHER = 32, /*!< Other (white) */
	OL_HEAD = 33, /*!< Head */
	OL_BODY = 34, /*!< Body */
	OL_SPINE1 = 35, /*!< Spine1 */
	OL_SPINE2 = 36, /*!< Spine2 */
	OL_L_UPPER_ARM = 37, /*!< LUpperArm */
	OL_L_FOREARM = 38, /*!< LForeArm */
	OL_L_HAND = 39, /*!< LHand */
	OL_L_THIGH = 40, /*!< LThigh */
	OL_L_CALF = 41, /*!< LCalf */
	OL_L_FOOT = 42, /*!< LFoot */
	OL_R_UPPER_ARM = 43, /*!< RUpperArm */
	OL_R_FOREARM = 44, /*!< RForeArm */
	OL_R_HAND = 45, /*!< RHand */
	OL_R_THIGH = 46, /*!< RThigh */
	OL_R_CALF = 47, /*!< RCalf */
	OL_R_FOOT = 48, /*!< RFoot */
	OL_TAIL = 49, /*!< Tail */
	OL_SIDE_WEAPON = 50, /*!< SideWeapon */
	OL_SHEILD = 51, /*!< Shield */
	OL_QUIVER = 52, /*!< Quiver */
	OL_BACK_WEAPON = 53, /*!< BackWeapon */
	OL_BACK_WEAPON2 = 54, /*!< BackWeapon (?) */
	OL_PONYTAIL = 55, /*!< PonyTail */
	OL_WING = 56, /*!< Wing */
	OL_NULL = 57, /*!< Null */
} OblivionLayer;

template <> std::string EnumToString<OblivionLayer>(OblivionLayer value);
template <> OblivionLayer StringToEnum<OblivionLayer>(const std::string& value);
void NifStream( OblivionLayer & val, istream& in, uint version = 0 );
void NifStream( OblivionLayer const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, OblivionLayer const & val );

/*!
 * The type of animation interpolation (blending) that will be used on
 * the associated key frames.
 */
typedef enum KeyType : uint {
	LINEAR_KEY = 1, /*!< Use linear interpolation. */
	QUADRATIC_KEY = 2, /*!< Use quadratic interpolation.  Forward and back tangents will be stored. */
	TBC_KEY = 3, /*!< Use Tension Bias Continuity interpolation.  Tension, bias, and continuity will be stored. */
	XYZ_ROTATION_KEY = 4, /*!< For use only with rotation data.  Separate X, Y, and Z keys will be stored instead of using quaternions. */
	UNKNOWN_KEY = 5, /*!< Unknown.  Step function? */
} KeyType;

template <> std::string EnumToString<KeyType>(KeyType value);
template <> KeyType StringToEnum<KeyType>(const std::string& value);
void NifStream( KeyType & val, istream& in, uint version = 0 );
void NifStream( KeyType const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, KeyType const & val );

/*!
 * An unsigned 32-bit integer, which describes how to apply vertex
 * colors.
 */
typedef enum VertMode : uint {
	VERT_MODE_SRC_IGNORE = 0, /*!< Source Ignore. */
	VERT_MODE_SRC_EMISSIVE = 1, /*!< Source Emissive. */
	VERT_MODE_SRC_AMB_DIF = 2, /*!< Source Ambient/Diffuse. (Default) */
} VertMode;

template <> std::string EnumToString<VertMode>(VertMode value);
template <> VertMode StringToEnum<VertMode>(const std::string& value);
void NifStream( VertMode & val, istream& in, uint version = 0 );
void NifStream( VertMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, VertMode const & val );

/*!
 * A material, used by havok shape objects.
 */
typedef enum HavokMaterial : uint {
	HAV_MAT_STONE = 0, /*!< Stone */
	HAV_MAT_CLOTH = 1, /*!< Cloth */
	HAV_MAT_DIRT = 2, /*!< Dirt */
	HAV_MAT_GLASS = 3, /*!< Glass */
	HAV_MAT_GRASS = 4, /*!< Grass */
	HAV_MAT_METAL = 5, /*!< Metal */
	HAV_MAT_ORGANIC = 6, /*!< Organic */
	HAV_MAT_SKIN = 7, /*!< Skin */
	HAV_MAT_WATER = 8, /*!< Water */
	HAV_MAT_WOOD = 9, /*!< Wood */
	HAV_MAT_HEAVY_STONE = 10, /*!< Heavy Stone */
	HAV_MAT_HEAVY_METAL = 11, /*!< Heavy Metal */
	HAV_MAT_HEAVY_WOOD = 12, /*!< Heavy Wood */
	HAV_MAT_CHAIN = 13, /*!< Chain */
	HAV_MAT_SNOW = 14, /*!< Snow */
	HAV_MAT_STONE_STAIRS = 15, /*!< Stone Stairs */
	HAV_MAT_CLOTH_STAIRS = 16, /*!< Cloth Stairs */
	HAV_MAT_DIRT_STAIRS = 17, /*!< Dirt Stairs */
	HAV_MAT_GLASS_STAIRS = 18, /*!< Glass Stairs */
	HAV_MAT_GRASS_STAIRS = 19, /*!< Grass Stairs */
	HAV_MAT_METAL_STAIRS = 20, /*!< Metal Stairs */
	HAV_MAT_ORGANIC_STAIRS = 21, /*!< Organic Stairs */
	HAV_MAT_SKIN_STAIRS = 22, /*!< Skin Stairs */
	HAV_MAT_WATER_STAIRS = 23, /*!< Water Stairs */
	HAV_MAT_WOOD_STAIRS = 24, /*!< Wood Stairs */
	HAV_MAT_HEAVY_STONE_STAIRS = 25, /*!< Heavy Stone Stairs */
	HAV_MAT_HEAVY_METAL_STAIRS = 26, /*!< Heavy Metal Stairs */
	HAV_MAT_HEAVY_WOOD_STAIRS = 27, /*!< Heavy Wood Stairs */
	HAV_MAT_CHAIN_STAIRS = 28, /*!< Chain Stairs */
	HAV_MAT_SNOW_STAIRS = 29, /*!< Snow Stairs */
	HAV_MAT_ELEVATOR = 30, /*!< Elevator */
} HavokMaterial;

template <> std::string EnumToString<HavokMaterial>(HavokMaterial value);
template <> HavokMaterial StringToEnum<HavokMaterial>(const std::string& value);
void NifStream( HavokMaterial & val, istream& in, uint version = 0 );
void NifStream( HavokMaterial const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, HavokMaterial const & val );

/*!
 * Specifies the pixel format used by the NiPixelData object to store a
 * texture.
 */
typedef enum PixelFormat : uint {
	PX_FMT_RGB8 = 0, /*!< 24-bit color: uses 8 bit to store each red, blue, and green component. */
	PX_FMT_RGBA8 = 1, /*!< 32-bit color with alpha: uses 8 bits to store each red, blue, green, and alpha component. */
	PX_FMT_PAL8 = 2, /*!< 8-bit palette index: uses 8 bits to store an index into the palette stored in a NiPallete object. */
} PixelFormat;

template <> std::string EnumToString<PixelFormat>(PixelFormat value);
template <> PixelFormat StringToEnum<PixelFormat>(const std::string& value);
void NifStream( PixelFormat & val, istream& in, uint version = 0 );
void NifStream( PixelFormat const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, PixelFormat const & val );

/*!
 * The animation cyle behavior.
 */
typedef enum CycleType : uint {
	CYCLE_LOOP = 0, /*!< Loop */
	CYCLE_REVERSE = 1, /*!< Reverse */
	CYCLE_CLAMP = 2, /*!< Clamp */
} CycleType;

template <> std::string EnumToString<CycleType>(CycleType value);
template <> CycleType StringToEnum<CycleType>(const std::string& value);
void NifStream( CycleType & val, istream& in, uint version = 0 );
void NifStream( CycleType const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, CycleType const & val );

/*!
 * An unsigned 32-bit integer, describing the apply mode of a texture.
 */
typedef enum ApplyMode : uint {
	APPLY_REPLACE = 0, /*!< Replaces existing color */
	APPLY_DECAL = 1, /*!< For placing images on the object like stickers. */
	APPLY_MODULATE = 2, /*!< Modulates existing color. (Default) */
	APPLY_HILIGHT = 3, /*!< PS2 Only.  Function Unknown. */
	APPLY_HILIGHT2 = 4, /*!< PS2 Only.  Function Unknown. */
} ApplyMode;

template <> std::string EnumToString<ApplyMode>(ApplyMode value);
template <> ApplyMode StringToEnum<ApplyMode>(const std::string& value);
void NifStream( ApplyMode & val, istream& in, uint version = 0 );
void NifStream( ApplyMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, ApplyMode const & val );

/*!
 * The force field's type.
 */
typedef enum FieldType : uint {
	FIELD_WIND = 0, /*!< Wind (fixed direction) */
	FIELD_POINT = 1, /*!< Point (fixed origin) */
} FieldType;

template <> std::string EnumToString<FieldType>(FieldType value);
template <> FieldType StringToEnum<FieldType>(const std::string& value);
void NifStream( FieldType & val, istream& in, uint version = 0 );
void NifStream( FieldType const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, FieldType const & val );

/*!
 * Determines the way the billboard will react to the camera.
 */
typedef enum BillboardMode : ushort {
	ALWAYS_FACE_CAMERA = 0, /*!< The billboard will always face the camera. */
	ROTATE_ABOUT_UP = 1, /*!< The billboard will only rotate around the up axis. */
	RIGID_FACE_CAMERA = 2, /*!< Rigid Face Camera. */
	ALWAYS_FACE_CENTER = 3, /*!< Always Face Center. */
	RIGID_FACE_CENTER = 4, /*!< Rigid Face Center. */
} BillboardMode;

template <> std::string EnumToString<BillboardMode>(BillboardMode value);
template <> BillboardMode StringToEnum<BillboardMode>(const std::string& value);
void NifStream( BillboardMode & val, istream& in, uint version = 0 );
void NifStream( BillboardMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, BillboardMode const & val );

/*!
 * The type of texture.
 */
typedef enum TexType : uint {
	BASE_MAP = 0, /*!< The basic texture used by most meshes. */
	DARK_MAP = 1, /*!< Used to darken the model with false lighting. */
	DETAIL_MAP = 2, /*!< Combined with base map for added detail.  Usually tiled over the mesh many times for close-up view. */
	GLOSS_MAP = 3, /*!< Allows the specularity (glossyness) of an object to differ across its surface. */
	GLOW_MAP = 4, /*!< Creates a glowing effect.  Basically an incandescence map. */
	BUMP_MAP = 5, /*!< Used to make the object appear to have more detail than it really does. */
	DECAL_0_MAP = 6, /*!< For placing images on the object like stickers. */
	DECAL_1_MAP = 7, /*!< For placing images on the object like stickers. */
} TexType;

template <> std::string EnumToString<TexType>(TexType value);
template <> TexType StringToEnum<TexType>(const std::string& value);
void NifStream( TexType & val, istream& in, uint version = 0 );
void NifStream( TexType const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, TexType const & val );

/*!
 * Specifies the availiable texture clamp modes.  That is, the behavior
 * of pixels outside the range of the texture.
 */
typedef enum TexClampMode : uint {
	CLAMP_S_CLAMP_T = 0, /*!< Clamp in both directions. */
	CLAMP_S_WRAP_T = 1, /*!< Clamp in the S(U) direction but wrap in the T(V) direction. */
	WRAP_S_CLAMP_T = 2, /*!< Wrap in the S(U) direction but clamp in the T(V) direction. */
	WRAP_S_WRAP_T = 3, /*!< Wrap in both directions. */
} TexClampMode;

template <> std::string EnumToString<TexClampMode>(TexClampMode value);
template <> TexClampMode StringToEnum<TexClampMode>(const std::string& value);
void NifStream( TexClampMode & val, istream& in, uint version = 0 );
void NifStream( TexClampMode const & val, ostream& out, uint version = 0  );
ostream & operator<<( ostream & out, TexClampMode const & val );

}
#endif
